#!/usr/local/bin/perl5 -w

# This program will setup a new collection from a model one. It does this by
# copying the model, moving files to have the correct names, and replacing
# text within the files to match the parameters.

BEGIN {
    die "GSDLHOME not set\n" unless defined $ENV{'GSDLHOME'};
    unshift (@INC, "$ENV{'GSDLHOME'}/lib");
}

use parsargv;

sub print_usage {
    print STDERR "\n  usage: $0 [options] collection-name\n\n";
    print STDERR "  options:\n";
    print STDERR "   -creator email      Your email address\n";
    print STDERR "   -maintainer email   The current maintainer's email address\n";
    print STDERR "   -public true|false  If this collection has anonymous access\n";
    print STDERR "   -beta true|false    If this collection is still under development\n";
    print STDERR "   -index type         The indexes which should be made\n";
    print STDERR "   -indextext name     The index description\n";
    print STDERR "   -defaultindex type  The index to use if no others are supplied\n\n";
}

sub traverse_dir
{
    my ($modeldir, $coldir) = @_;
    my ($newfile, @filetext);

    if (!(-e $coldir)) {
	if (!mkdir ($coldir, 0775)) {
	    die "$!";
	}
    }

    opendir(DIR, $modeldir) || die "Can't read $modeldir";
    my @files = grep(!/^\.\.?$/, readdir(DIR));
    closedir(DIR);

    foreach $file (@files)
    {
	if (-d "$modeldir/$file") {
	    traverse_dir ("$modeldir/$file", "$coldir/$file");

	} else {
	    my $destfile = $file;
	    $destfile =~ s/^\modelcol/$collection/;
	    $destfile =~ s/^\MODELCOL/$capcollection/;

	    print STDERR "doing replacements for $modeldir/$file\n";
	    open (INFILE, "$modeldir/$file") || 
		die "Can't read file $modeldir/$file";
	    open (OUTFILE, ">$coldir/$destfile") ||
		die "Can't create file $coldir/destfile";

	    while (defined ($line = <INFILE>)) {
		$line =~ s/\*\*collection\*\*/$collection/g;
		$line =~ s/\*\*COLLECTION\*\*/$capcollection/g;
		$line =~ s/\*\*creator\*\*/$creator/g;
		$line =~ s/\*\*maintainer\*\*/$maintainer/g;
		$line =~ s/\*\*public\*\*/$public/g;
		$line =~ s/\*\*beta\*\*/$beta/g;
		$line =~ s/\*\*key\*\*/$key/g;
		$line =~ s/\*\*indexes\*\*/$indexesstr/g;
		$line =~ s/\*\*indexestext\*\*/$indexestextstr/g;
		$line =~ s/\*\*defaultindex\*\*/$defaultindex/g;

		print OUTFILE $line;
	    }
	    
	    close (OUTFILE);
	    close (INFILE);
	}
    }
}


my (@indexes, @indexestext);

# get and check options
if (!&parsargv::parse(\@ARGV,
		       'creator/\w+\@[\w\.]+/', \$creator,
		       'maintainer/\w+\@[\w\.]+/', \$maintainer,
		       'public/true|false/true', \$public,
		       'beta/true|false/true', \$beta,
		       'index/.*/document:all', \@indexes,
		       'indextext/\.*/Terms must appear within the same document', \@indexestext,
		       'defaultindex/.*/document:all', \$defaultindex
		       )) {
    &print_usage();
    die "\n";
}
    
# get and check the collection name
($collection) = @ARGV;
if (!defined($collection)) {
    &print_usage();
    die "\n";
}

if (length($collection) > 8) {
    print STDERR "The collection name must be less than 8 characters\n";
    print STDERR "so compatibility with earlier filesystems can be\n";
    print STDERR "maintained.\n";
    die "\n";
}

if ($collection eq "modelcol") {
    print STDERR "No collection can be named modelcol as this is the\n";
    print STDERR "name of the model collection.\n";
    die "\n";
}

if (!defined($creator) || $creator eq "") {
    print STDERR "The creator was not defined. This variable is\n";
    print STDERR "needed to recognise duplicate collection names.\n";
    die "\n";
}

if (!defined($maintainer) || $maintainer eq "") {
    $maintainer = $creator;
}

$public = "true" unless defined $public;
$beta = "true" unless defined $beta;

srand(time);
$key = int(rand()*100000000.0) + int(rand()*100000000.0);

# get capitalised version of the collection
$capcollection = $collection;
$capcollection =~ tr/a-z/A-Z/;


# get the strings to include.
$indexesstr = join ("\t", @indexes);
$indexestextstr = "";
for ($i=0; $i<scalar(@indexestext); $i++) {
    $indexestextstr .= "_$indexes[$i]_[] {$indexestext[$i]}\n";
}


# make sure the model collection exists
die "Cannot find the model collection $ENV{'GSDLHOME'}/collect/modelcol" unless
    (-d "$ENV{'GSDLHOME'}/collect/modelcol");

# make sure this collection does not already exist
if (-e "$ENV{'GSDLHOME'}/collect/$collection") {
    print STDERR "This collection already exists\n";
    die "\n";
}


# start creating the collection
print STDERR "Creating the collection $collection\n\n";
&traverse_dir ("$ENV{'GSDLHOME'}/collect/modelcol", "$ENV{'GSDLHOME'}/collect/$collection");
print STDERR "\n\nThe new collection is in $ENV{'GSDLHOME'}/collect/$collection.\n\n";

